/**
 * Tomislav Podhraški
 * http://www.justpinegames.com
 */

package  
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	import starling.display.Image;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.events.Event;
	import starling.textures.Texture;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.utils.Color;


	public class CharacterCreator extends Sprite
	{
		[Embed(source="../../FinalBackground.png")]
		private var BackgroundBitmap:Class;
		
		[Embed(source="../../FinalCharacter.png")]
		private var CharacterBitmap:Class;
		
		[Embed(source="../../InitialColor.png")]
		private var InitialPaletteBitmap:Class;
				
		private const SECTION_NONE:int = 0;
		private var _sectionSelection:int;
		
		private var _debugInterface:Interface;
		
		private var _chracterBitmapData:BitmapData;
		private var _characterImage:PalettedImage;
		
		private var _basePaletteVector:Vector.<Number>;
		private var _paletteVector:Vector.<uint>;
		
		private var _baseColors:Vector.<uint>;
		
		public function CharacterCreator() 
		{
			_sectionSelection = SECTION_NONE;
			
			// Default colors
			_baseColors = new Vector.<uint>(7, true);
			_baseColors[0] = 0x746f59; // armor
			_baseColors[1] = 0x522f1e; // boots
			_baseColors[2] = 0xa42222; // shirt
			_baseColors[3] = 0xe10000; // pants/gloves
			_baseColors[4] = 0xf9b385; // skin
			_baseColors[5] = 0xf4b92a; // hair
			_baseColors[6] = 0xcac5c5; // eyes
			
			var paletteBitmap:Bitmap = new InitialPaletteBitmap();
			
			// vectors that store base palette values
			_basePaletteVector = new Vector.<Number>(256, true);
			_paletteVector = new Vector.<uint>(256, true);
			for (var i:int = 0; i < paletteBitmap.bitmapData.width; i++)
			{
				_paletteVector[i] = paletteBitmap.bitmapData.getPixel(i, 0);
				
				var paletteValue:Number = Color.getRed(_paletteVector[i]);
				_basePaletteVector[i] = paletteValue / 255.0;
			}
			
			// Create and add a background image
			var backgroundImage:Image = new Image(Texture.fromBitmap(new BackgroundBitmap()));
			this.addChild(backgroundImage);
						
			var bitmap:Bitmap = new CharacterBitmap();
			_chracterBitmapData = bitmap.bitmapData;
			
			var mainTexture:Texture = Texture.fromBitmap(bitmap);
			var palette:Texture = Texture.fromBitmap(paletteBitmap);
			
			// character image using palette image class
			_characterImage = new PalettedImage(mainTexture, recalculatePalette());
			_characterImage.x = 50;
			_characterImage.y = 25;
			this.addChild(_characterImage);
			_characterImage.addEventListener(TouchEvent.TOUCH, imageTouched);
			
			// color picker interface
			_debugInterface = new Interface();
			_debugInterface.addEventListener(Interface.COLOR_CHANGED, colorChanged);
			this.addChild(_debugInterface);
			
			// loop used to animated selected element
			this.addEventListener(EnterFrameEvent.ENTER_FRAME, enterFrame);
		}
		
		private function textureFromVectorPalette(palette:Vector.<uint>):Texture
		{
			// create a bitmap data with vector of colors
			var paletteBitmapData:BitmapData = new BitmapData(palette.length, 1, false);
			for (var i:int = 0; i < palette.length; i++)
			{
				paletteBitmapData.setPixel(i, 0, palette[i]);
			}
			
			// upload the data to texture
			var paletteTexture:Texture = Texture.fromBitmapData(paletteBitmapData);
			return paletteTexture;
		}
		
		private function enterFrame(e:EnterFrameEvent):void 
		{
			if (_sectionSelection != SECTION_NONE) {
				
				
				// start of the secton
				var start:int = 42 * (_sectionSelection - 1);
				
				// sen of the section
				var end:int = Math.min(254, 42 * _sectionSelection);

				// save the starting value of the palette section
				var tmp:int = _paletteVector[start];
				
				// go trough the section segment of the palette and sift the values to the left
				for (var i:int = start; i < end; i++) {
					_paletteVector[i] = _paletteVector[i + 1];
				}
				
				// use the saved staring value, wrapping around
				_paletteVector[end] = tmp;
				
				_characterImage.palette = textureFromVectorPalette(_paletteVector);
				
			}
		}
		
		private function imageTouched(e:TouchEvent):void 
		{
			var touchStarted:Touch = e.getTouch(_characterImage, TouchPhase.BEGAN);
			
			// If touch/mouse started or moved find the new location
			if (touchStarted) {
				var touchPoint:Point = touchStarted.getLocation(_characterImage);
				var characterColor:uint = _chracterBitmapData.getPixel(touchPoint.x, touchPoint.y);
				
				// take the red value
				var characterValue:uint = Color.getRed(characterColor);
				
				// 255 means transparent so we'll use that as deselection
				if (characterValue == 255) {
					_sectionSelection = SECTION_NONE;
				}
				else {
					// calculate the section, each section takes 42 pixels of the palette
					_sectionSelection = int(characterValue / 42) + 1;
				}
				
				_characterImage.palette = recalculatePalette();
			}
		
		}
		
		private function recalculatePalette():Texture 
		{
			// go through the palette
			for (var i:int = 0; i < _paletteVector.length; i++)
			{
				// 42 is the length of a segment in the palette
				var color:uint = _baseColors[int(i / 42)];
				
				// extract the RGB values from the segment color value and multiply original grayscale palette
				var r:uint = Color.getRed(color) * _basePaletteVector[i];
				var g:uint = Color.getGreen(color) * _basePaletteVector[i];
				var b:uint = Color.getBlue(color) * _basePaletteVector[i];
				
				// create a new palette color by joining color components
				_paletteVector[i] = Color.rgb(r, g, b);	
			}
			
			return textureFromVectorPalette(_paletteVector);
		}
		
		private function colorChanged(e:Event):void
		{
			var newColor:uint = e.data.color;
			
			// if section is selected change it's color and update the palette texture
			if (_sectionSelection != SECTION_NONE)
			{
				_baseColors[_sectionSelection - 1] = newColor;
				_characterImage.palette = recalculatePalette();
			}
		}
	}
}