/**
 * Tomislav Podhraški
 * http://www.justpinegames.com
 */

// Interface.as handles graphical interface for debugging.

package  
{
	import feathers.controls.Button;
	import feathers.controls.HSlider;
	import feathers.controls.ScrollContainer;
	import feathers.controls.Slider;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.DisplayListWatcher;
	import feathers.core.ITextRenderer;
	import flash.geom.Point;
	import starling.display.Image;
	import feathers.layout.VerticalLayout;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import starling.display.DisplayObject;
	import flash.text.TextFormat;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.core.Starling;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.textures.Texture;

	public class Interface extends Sprite
	{
		public static const COLOR_CHANGED:String = "colorChanged";
		
		// Simple image with most of the colors
		[Embed(source="../../FinalColorPicker.png")]
		private var ColorPicker:Class;
		
		private var _colorPickerBitmapData:BitmapData;
		private var _colorPickerImage:Image;
		
		private var _colorPreview:Quad;
		
		public function Interface() 
		{
			// helper message explaining what the demo does.
			var message:TextFieldTextRenderer = new TextFieldTextRenderer();
			message.textFormat = new TextFormat("Verdana", 12, 0xffffff);
			message.text = "Select part of the character\nfor which the color\nwill be changed.";
			message.x = 406;
			message.y = 6;
			this.addChild(message);
			
			
			// Bitmap data that will be used to find out colors at points
			var colorPickerBitmap:Bitmap = new ColorPicker();
			_colorPickerBitmapData = colorPickerBitmap.bitmapData;
			
			// Image to display the colors
			_colorPickerImage = new Image(Texture.fromBitmap(colorPickerBitmap));
			_colorPickerImage.x = 405;
			_colorPickerImage.y = 75;
			this.addChild(_colorPickerImage);
			_colorPickerImage.addEventListener(TouchEvent.TOUCH, touchEvent);
			
			// Current color selection
			_colorPreview = new Quad(80, 80, 0x777777);
			_colorPreview.x = 460;
			_colorPreview.y = 290;
			this.addChild(_colorPreview);

		}
		
		
		private function touchEvent(e:TouchEvent):void
		{
			var touchStarted:Touch = e.getTouch(_colorPickerImage, TouchPhase.BEGAN);
			var touchMoved:Touch = e.getTouch(_colorPickerImage, TouchPhase.MOVED);
			
			var newColorPoint:Point;
			
			// If touch/mouse started or moved find the new location
			if (touchStarted)
			{
				newColorPoint = touchStarted.getLocation(_colorPickerImage);
			}
			if (touchMoved)
			{
				newColorPoint = touchMoved.getLocation(_colorPickerImage);
			}
			
			// If location is found, calculate the color at that spot from the bitmapdata
			if (newColorPoint)
			{
				// limit the location to the image size (fixes the issue when mouse is dragged of the image)
				var colorX:int = Math.min(189, Math.max(0, Math.floor(newColorPoint.x)));
				var colorY:int = Math.min(189, Math.max(0, Math.floor(newColorPoint.y)));
				
				var newColor:uint = _colorPickerBitmapData.getPixel(colorX, colorY);
				dispatchEvent(new Event(COLOR_CHANGED, false, { color: newColor } ));
				_colorPreview.color = newColor;
			}
		}
		
	}
}